var game, ball, player, ballOwner, timer, ai, arena,
		userAim = 0, 
		score = [0,0],
		scoreboard, up, down, left, right;

function preloadSprites() {
	// Since our sprites vary in size, we're going to manually select the bounding box
	// for each sprite in the sheet as we create it.
	game.load.image('sprites', 'sprites.png');
	
	game.load.image("arena", "arena.png?");
}

function createGame() {
	game.physics.startSystem(Phaser.Physics.ARCADE);
	
	arena = game.add.sprite(0, 0, 'arena');
	arena.width = innerWidth;
	arena.height = innerHeight;
	arena.inputEnabled = true;
	arena.events.onInputDown.add(function() {
		fire(userAim);
	})
	
	player = game.add.sprite(0, innerHeight / 2, 'sprites');
	player.cropRect = new Phaser.Rectangle(0, 128, 128, 128);
	player.updateCrop();
	
	game.physics.enable(player, Phaser.Physics.ARCADE);
	player.body.immovable = true;
	
	opponent = game.add.sprite(innerWidth - 128, innerHeight / 2, 'sprites');
	opponent.cropRect = new Phaser.Rectangle(0, 0, 128, 128);
	opponent.updateCrop();
	
	ball = game.add.sprite(innerWidth / 2, innerHeight / 2, 'sprites');
	ball.cropRect = new Phaser.Rectangle(128, 128, 64, 64);
	ball.updateCrop();
	
	
	game.physics.enable(ball, Phaser.Physics.ARCADE);
  ball.body.collideWorldBounds = true;
  ball.body.bounce.set(1);
	ball.body.allowGravity = false;	
	ballOwner = player;
	
	game.physics.enable(opponent, Phaser.Physics.ARCADE);
	opponent.body.immovable = true;
	
	scoreboard = game.add.text('0 - 0', {font: 'bold 30px'});
	scoreboard.fill = 'white';
	scoreboard.font = 'PressStart2P';
	scoreboard.anchor.x = 0.5;
	scoreboard.position.x = innerWidth / 2;
	scoreboard.position.y = 10;
	
	up = game.add.sprite(16, innerHeight - 128 - 32, 'sprites');
	up.cropRect = new Phaser.Rectangle(128, 0, 64, 64);
	up.updateCrop();
	up.inputEnabled = true;
	up.events.onInputDown.add(function() {
		player.position.y -= 30;
		if (player.position.y < 0) player.position.y = 0;
	});
	
	down = game.add.sprite(16, innerHeight - 64 - 16, 'sprites');
	down.cropRect = new Phaser.Rectangle(128, 64, 64, 64);
	down.updateCrop();
	down.inputEnabled = true;
	down.events.onInputDown.add(function() {
		player.position.y += 30;
		if (player.position.y > innerHeight - 128) player.position.y = innerHeight - 128;
	});
	
	left = game.add.sprite(innerWidth - 64 - 16, innerHeight - 128 - 32, 'sprites');
	left.cropRect = new Phaser.Rectangle(196, 0, 64, 64);
	left.updateCrop();
	left.inputEnabled = true;
	left.events.onInputDown.add(function() {
		userAim = -1;
	});
	left.events.onInputUp.add(function() {
		userAim = 0;
	});
	
	right = game.add.sprite(innerWidth - 64 - 16, innerHeight - 64 - 16, 'sprites');
  right.cropRect = new Phaser.Rectangle(192, 64, 64, 64);
	right.updateCrop();
	right.inputEnabled = true;
	right.events.onInputDown.add(function() {
		userAim = 1;
	});
	right.events.onInputUp.add(function() {
		userAim = 0;
	});
	
	game.input.keyboard.onDownCallback = function(key) {
		switch (String.fromCharCode(key.keyCode).toUpperCase()) {
				case 'A':
					player.y -= 30;
					if (player.y < 0) player.y = 0;
					break;
				case 'Z':
					player.y += 30;
					if (player.y > innerHeight - 128) player.y = innerHeight - 128;
					break;
				case 'K':
					userAim = -1;
					break;
				case 'M':
					userAim = 1;
					break;
				case ' ':
					fire(userAim);
					break;
		}
	}
	
	game.input.keyboard.onUpCallback = function(key) {
		switch (String.fromCharCode(key.keyCode).toUpperCase()) {
			case 'K':
			case 'M':
				userAim = 0;
				break;
		}
	};
	
	ai = new AI(opponent);	
}

function fire(aim) {
	var v = [0,0];
	if (ballOwner == player) {
  	switch(aim) {
		case -1:
			v = [707, -707];
			break;
		case 0:
			v = [1000,0];
			break;
		case 1:
			v = [707, 707];
		}
		ball.body.velocity.setTo(v[0],v[1]);  
		ballOwner = undefined;
	} else if (ballOwner == opponent) {
		switch(aim) {
		case -1:
			v = [-707, -707];
			break;
		case 0:
			v = [-1000,0];
			break;
		case 1:
			v = [-707, 707];
		}
		ball.body.velocity.setTo(v[0],v[1]);  
		ballOwner = undefined;
	}
}

function update() {
	if (ballOwner != undefined) {
		var x,y;
		y = ballOwner.y + 32;
		if (ballOwner == player) {
			x = ballOwner.x + 100;
		} else {
			x = ballOwner.x - 32;
		}
		ball.position.x = x;
		ball.position.y = y;
	} else {
		game.physics.arcade.collide(player, ball, function() {
			ballOwner = player;
		});
		game.physics.arcade.collide(opponent, ball, function() {
			ballOwner = opponent;
		});
	}
	
	ai.update();
}

function AI(playerToControl) {
  var ctl = playerToControl;
  var State = {
    WAITING: 0,
    FOLLOWING: 1,
    AIMING: 2
  }
  var currentState = State.FOLLOWING;

  function aimAndFire() {
    // We'll repeat the motion action 5 to 10 times
    var numRepeats = Math.floor(5 + Math.random() * 5);

    function randomMove() {
      if (Math.random() > .5) {
        this.y -= 30;
				if (this.y < 0) this. y = 0;
      } else {
        this.y += 30;
				if (this.y > innerHeight - 128) this.y = innerHeight - 128;
      }
    }

    function randomAimAndFire() {
      var d = Math.floor( Math.random() * 3 - 1 );
      fire(d);
      currentState = State.FOLLOWING;
    }

		game.time.events.repeat(250, numRepeats, randomMove, ctl);
		game.time.events.add(250 * (numRepeats + 1), randomAimAndFire, ctl);
  }

  function moveTowardsBall() {
    if(ball.position.y <= ctl.position.y + 64) {
    	ctl.y -= 30;
			if (ctl.y < 0) ctl.y = 0;
    } else {
      ctl.y += 30;
			if (ctl.y > innerHeight - 128) ctl.y = innerHeight - 128;
    }
		
    setTimeout(function() {
      currentState = State.FOLLOWING;
    }, 400);
  }

  function update() {
		scoreboard.setText(score[0] + ' - ' + score[1]);
    switch (currentState) {
      case State.FOLLOWING:
				if (ballOwner == ctl) { 
          currentState = State.AIMING;
					aimAndFire();
        } else {
					moveTowardsBall();
        	currentState = State.WAITING;
				}
      case State.WAITING:
        break;
      case State.AIMING:
        break;
    }
		
		if (ball.body.onWall()) {
			if (ball.position.x > innerWidth / 2){
				console.log('player scored!');
  			score[0]++;
  			ballOwner = opponent;
			} else {
				console.log('opponent scored!');
  			score[1]++;
  			ballOwner = player;
			}
		}
  }

  return {
    update: update
  }
}

game = new Phaser.Game(
		innerWidth, innerHeight, Phaser.AUTO, '', 
		{ preload: preloadSprites, create: createGame, update: update});
