var lastUpdate = 0;
var player, ball, opponent, ai, arena;
var distance = 24;
var score  = [0, 0];

var Ball = function (stage) {
	var velocity = [0, 0];
	var position = [0, 0];
	var element = new PIXI.Sprite.fromFrame(2);
	element.anchor.x = 0.5;
	element.anchor.y = 0.5;
	stage.addChild(element);
  var owner;
  var halfTile = 32;
  var paused = false;

	var move = function (t) {
		if (owner !== undefined) {
      var ownerPosition = owner.getPosition();
			element.position.y = ownerPosition.y;
      if (owner.getSide() === 'left') {
        element.position.x = ownerPosition.x + owner.getSize() - 20;
      } else {
        element.position.x = ownerPosition.x - owner.getSize() + 20;
      }
		} else {
			if (element.position.y  <= 0 ||
				  element.position.y  >= innerHeight) {
				velocity[1] = -velocity[1];
			}
			element.position.x += velocity[0] * t; 
			element.position.y += velocity[1] * t; 	
		}	
	};
  
  function checkScored() {
		if (element.position.x <= 0) {
      pause();
      // Opponent scored!
			$(document).trigger('ping:opponentScored');
		}

		if (element.position.x >= innerWidth) {
      pause();
      // Player scored!
			$(document).trigger('ping:playerScored');
		}
	}

	var update = function(t) {
    // First we handle motion of the ball
    if(!paused) {
      move(t);
    }

    // The ball is under control of a player, so no need to update
    if (owner !== undefined) {
      return;
    }

    // First we check if the ball is about to be grabbed by the player
    var playerPosition = player.getPosition();
    if (element.position.x <= player.getSize() &&
        element.position.y >= playerPosition.y  &&
        element.position.y <= playerPosition.y + player.getSize()) {
      console.log("Grabbed by player!");
      owner = player;
    }


    // Then the opponent...
    var opponentPosition = opponent.getPosition();
     if (element.position.x >= innerWidth - opponent.getSize() &&
        element.position.y >= opponentPosition.y  &&
        element.position.y <= opponentPosition.y + opponent.getSize()) {
      console.log("Grabbed by opponent!");
      owner = opponent;
    }

    checkScored();
	}

  function pause() {
    paused = true;
  }

  function start() {
    paused = false;
  }

  return {
    update: update,
    pause:        pause,
    start:        start,
    getOwner:     function()  { return owner; },
    setOwner:     function(o) { owner = o; },
    getVelocity:  function()  { return velocity }, 
    setVelocity:  function(v) { velocity = v; },
    getPosition:  function() { return element.position; },
  }
};

var Player = function (side, stage) {
  var aim = 0;
  var tileSize = 128;
  var element;
	if (side  == 'right') {
		element = new PIXI.Sprite.fromFrame(1);
	} else {
		element = new PIXI.Sprite.fromFrame(0);
	}
	element.anchor.x = 0.5;
	element.anchor.y = 0.5;
	stage.addChild(element);

  var move = function(y) {
    element.position.y += y;
    if (element.position.y <= tileSize/2)  {
      element.position.y = tileSize/2;
    }

    if (element.position.y >= innerHeight - tileSize/2) {
      element.position.y = innerHeight - tileSize/2;
    }

    if (side == 'right') {
      element.position.x = innerWidth - tileSize/2;
    } else {
			element.position.x = tileSize/2;
		}
  }

  var fire = function() {
    if (ball.getOwner() !== this) {
      return;
    }

    var v = [0,0];

    if (side == 'left') {
      switch(aim) {
      case -1:
        v = [.707, -.707];
        break;
      case 0:
        v = [1,0];
        break;
      case 1:
        v = [.707, .707];
      }
    } else {
      switch(aim) {
      case -1:
        v = [-.707, -.707];
        break;
      case 0:
        v = [-1,0];
        break;
      case 1:
        v = [-.707, .707];
      }
    }
    ball.setVelocity(v);
    ball.setOwner(undefined);
  }

  return {
    move: move,
    fire: fire,
    getSide:      function()  { return side; },
    setAim:       function(a) { aim = a; },
    getPosition:  function()  { return element.position; },
    getSize:      function()  { return tileSize; }
  }
};

function AI(playerToControl) {
  var ctl = playerToControl;
  var State = {
    WAITING: 0,
    FOLLOWING: 1,
    AIMING: 2
  }
  var currentState = State.FOLLOWING;

  function repeat(cb, cbFinal, interval, count) {
    var timeout = function() {
      repeat(cb, cbFinal, interval, count-1);
    }
    if (count <= 0) {
      cbFinal();
    } else {
      cb();
      setTimeout(function() {
        repeat(cb, cbFinal, interval, count-1);
      }, interval);
    }
  }

  function aimAndFire() {
    // We'll repeat the motion action 5 to 10 times
    var numRepeats = Math.floor(5 + Math.random() * 5);

    function randomMove() {
      if (Math.random() > .5) {
        ctl.move(-distance);
      } else {
        ctl.move(distance);
      }
    }

    function randomAimAndFire() {
      var d = Math.floor( Math.random() * 3 - 1 );
      opponent.setAim(d);
      opponent.fire();

      // Finally, set the state to FOLLOWING
      currentState = State.FOLLOWING;
    }

    repeat(randomMove, randomAimAndFire, 250, numRepeats);
  }

  function moveTowardsBall() {
    if(ball.getPosition().y >= ctl.getPosition().y + ctl.getSize()/2) {
      ctl.move(distance);
    } else {
      ctl.move(-distance);
    }
    setTimeout(function() {
      currentState = State.FOLLOWING;
    }, 400);
  }

  function update() {
    switch (currentState) {
      case State.FOLLOWING:
				if (ball.getOwner() === ctl) { 
          currentState = State.AIMING;
					aimAndFire();
        } else {
					moveTowardsBall();
        	currentState = State.WAITING;
				}
      case State.WAITING:
        break;
      case State.AIMING:
        break;
    }
  }

  return {
    update: update
  }
}

var stage, render, scoreboard;
var up, down, left, right;

function update(time) {
  var t = time - lastUpdate;
  lastUpdate = time;

  ball.update(t);
  ai.update();
	
	// Forces Pixi to use the new font once it's loaded.
	var newScore = score[0] + " - " + score[1];
  scoreboard.setText(newScore);
	
	renderer.render(stage);

  requestAnimationFrame(update);
}

function startGame() {
	
	arena = new PIXI.Sprite.fromImage('arena.png?');
	arena.width = innerWidth;
	arena.height = innerHeight;
	arena.interactive = true;
	stage.addChild(arena);
	arena.touchstart = arena.mousedown = function(e) {
		if (e.target == arena) {
			player.fire();
		}
	}
	
	player = Player('left', stage);
  player.move(0);
  opponent = Player('right', stage);
  opponent.move(0);
  ball = Ball(stage);
  ai = AI(opponent);
	ball.setOwner(player);
	
	
	scoreboard = new PIXI.Text('0 - 0', {font: 'bold 30px PressStart2P'});
	scoreboard.anchor.x = 0.5;
	scoreboard.position.x = innerWidth / 2;
	scoreboard.position.y = 10;
 	stage.addChild(scoreboard);
	
	up = new PIXI.Sprite.fromFrame(3);
	up.interactive = true;
	up.position.x = 16;
	up.position.y = innerHeight - 128 - 32;
	stage.addChild(up);
	up.touchstart = up.mousedown = function() {
		player.move(-distance);
	}
	
	down = new PIXI.Sprite.fromFrame(4);
	down.interactive = true;
	down.position.x = 16;
	down.position.y = innerHeight - 64 - 16;
	stage.addChild(down);
	down.touchstart = down.mousedown = function() {
		player.move(distance);
	}
	
	left = new PIXI.Sprite.fromFrame(5);
	left.interactive = true;
	left.position.x = innerWidth - 64 - 16;
	left.position.y = innerHeight - 128 - 32;
	stage.addChild(left);
	left.touchstart = left.mousedown = function() {
		player.setAim(-1);
	}
	
	right = new PIXI.Sprite.fromFrame(6);
	right.interactive = true;
	right.position.x = innerWidth - 64 - 16;
	right.position.y = innerHeight - 64 - 16;
	stage.addChild(right);
	right.touchstart = right.mousedown = function() {
		player.setAim(1);
	}
	
	left.touchend = left.mouseup = right.touchend = right.mouseup = 
	function() {
		player.setAim(0);
	};
	
	requestAnimationFrame(update);
}

$(document).ready(function() {
	// create an new instance of a pixi stage
 	stage = new PIXI.Stage(0xFFFFFFFF);
	lastUpdate = 0;

  // create a renderer instance.
  renderer = PIXI.autoDetectRenderer(innerWidth, innerHeight);
  document.body.appendChild(renderer.view);
	
	var images = ["sprites.json"];
  var loader = new PIXI.AssetLoader(images);
	loader.onComplete = startGame;
  loader.load(); 
});

$(document).keydown(function(event) {
  var event = event || window.event;
  switch(String.fromCharCode(event.keyCode).toUpperCase()) {
    case 'A':
      player.move(-distance);
      break;
    case 'Z':
      player.move(distance);
      break;
    case 'K':
      player.setAim(-1);
      break;
    case 'M':
      player.setAim(1);
      break;
    case ' ':
      player.fire();
      break;
  }

  return false;
});

$(document).keyup(function(event) {
  var event = event || window.event;
  switch(String.fromCharCode(event.keyCode).toUpperCase()) {
    case 'K':
    case 'M':
      player.setAim(0);
      break;
  }

  return false;
});

$(document).on('ping:playerScored', function(e) {
  console.log('player scored!');
  score[0]++;
  ball.setOwner(opponent);
  ball.start();
});

$(document).on('ping:opponentScored', function(e) {
  console.log('opponent scored!');
  score[1]++;
  var newScore = score[0] + " - " + score[1]
  scoreboard.setText(newScore);
  ball.setOwner(player);
  ball.start();
});
